unit FormProxy;

// written by Ralf Steinhaeusser, 2000
// email : ralfiii@gmx.net

// FormProxy saves time for creation of complicated Forms that are not necessarily needed
//
// Forms are created as soon as they are REALLY needed !
// (Only applicable if only Show/Close/.... - properties, that means
//  Properties that are encapsulated in the TForm-class are needed externaly) 
//
// Usage :
// exchange lines in dpr-file for Forms that should not be created at startup
//    old : Application.CreateForm(TForm2, Form2);
//    new : TFormProxy.CreateForm(TForm2, Form2);
// in corresponding Forms-Units :
//    old : var Form2: TForm2;
//    new : var Form2: TFormProxy; (add unit FormProxy to uses-clause also)


interface

uses forms, Classes;

type TFormProxy = class (TComponent)
  private
      FInstanceClass : TComponentClass;
      Reference : TForm;
    Procedure NowNeeded;  
  protected
  public
    class procedure CreateForm(FormType : TComponentClass; var ProxyVariable : TFormProxy);
    constructor Create(AOwner: TComponent); override;

    // Imitation functions
    Procedure Show;
    Function ShowModal : integer;
    Procedure Update;
    Procedure Close;
    Procedure Hide;
  published
  end;

implementation

constructor TFormProxy.Create (AOwner: TComponent);
begin
     inherited Create (AOwner);

     FInstanceClass:=TForm;
     Reference:=nil;
end;


class procedure TFormProxy.CreateForm( FormType: TComponentClass;
                                       var ProxyVariable: TFormProxy);
begin
     ProxyVariable:=TFormProxy.Create (Application);
     TFormProxy(ProxyVariable).FInstanceClass:=FormType;
end;

procedure TFormProxy.NowNeeded;
var Instance: TComponent;
begin
     if Reference=nil
     then
     begin
          Instance := TComponent(FInstanceClass.NewInstance);
          TComponent(Reference) := Instance;
          try
             Instance.Create(Owner);
          except
             TComponent(Reference) := nil;
             raise;
          end;
     end;
end;

// ***********************
// Imitating functions
// ***********************

procedure TFormProxy.Show;
begin
     NowNeeded;
     Reference.Show;
end;

function TFormProxy.ShowModal: integer;
begin
     NowNeeded;
     result:=Reference.ShowModal;
end;

procedure TFormProxy.Update;
begin
     NowNeeded;
     Reference.Update;
end;

procedure TFormProxy.Close;
begin
     NowNeeded;
     Reference.Close;
end;


procedure TFormProxy.Hide;
begin
     NowNeeded;
     Reference.Hide;
end;

end.
